@@ -6,8 +6,9 @@
 from __future__ import print_function, division, unicode_literals
 
 from .omas_core import ODS
-from collections import Sequence
 import numpy
+import sys
+from collections.abc import Sequence
 
 try:
     _pyhdc_import_excp = None
