@@ -10,10 +10,18 @@ from __future__ import print_function, division, unicode_literals
 # --------------------------------------------
 import os
 import sys
+
+__version__ = open(os.path.abspath(str(os.path.dirname(__file__)) + os.sep + 'version'), 'r').read().strip()
+
+if sys.version_info < (3, 5):
+    raise Exception('''
+OMAS v%s only runs with Python 3.6+ and you are running Python %s
+''' % (__version__, '.'.join(map(str, sys.version_info[:2]))))
+
 import glob
 import json
 import copy
-from collections import MutableMapping, OrderedDict
+from collections import OrderedDict
 import re
 import numpy
 from pprint import pprint
@@ -21,6 +29,8 @@ from io import StringIO
 from contextlib import contextmanager
 import tempfile
 import warnings
+from functools import wraps
+import ast
 
 formatwarning_orig = warnings.formatwarning
 warnings.formatwarning = lambda message, category, filename, lineno, line=None: \
@@ -40,53 +50,22 @@ import uncertainties
 import uncertainties.unumpy as unumpy
 from uncertainties.unumpy import nominal_values, std_devs, uarray
 from uncertainties import ufloat
-import ast
 
 # xarrays: avoid loading xarrays upfront since it can be slow and it is not always used
 import xarray
 
-# Python3/2 import differences
-if sys.version_info < (3, 0):
-    import cPickle as pickle
+from collections.abc import MutableMapping
+import pickle
 
 
-    def b2s(string):
-        return string
-
-else:
-    basestring = str
-    unicode = str
-    import pickle
-
-    _orig_pickle_loads = pickle.loads
-
-
-    def _pickle_loads_python2compatible(*args, **kw):
-        kw.setdefault('encoding', 'latin1')
-        return _orig_pickle_loads(*args, **kw)
-
-
-    pickle.loads = _pickle_loads_python2compatible
-
-    _orig_pickle_load = pickle.load
-
-
-    def _pickle_load_python2compatible(*args, **kw):
-        kw.setdefault('encoding', 'latin1')
-        return _orig_pickle_load(*args, **kw)
-
-
-    pickle.load = _pickle_load_python2compatible
-
-
-    def b2s(bytes):
-        return bytes.decode("utf-8")
+def b2s(bytes):
+    return bytes.decode("utf-8")
 
 
 # --------------------------------------------
 # configuration of directories and IMAS infos
 # --------------------------------------------
-class IMAS_json_dir(unicode):
+class IMAS_json_dir(str):
     '''
     directory where the JSON data structures for the different versions of IMAS are stored
     '''
@@ -154,7 +133,7 @@ omas_rcparams.update({
     'allow_fake_imas_fallback': bool(int(os.environ.get('OMAS_ALLOW_FAKE_IMAS_FALLBACK', '0'))),
     'default_imas_version': _default_imas_version,
     'default_mongo_server': 'mongodb+srv://{user}:{pass}@omasdb-xymmt.mongodb.net',
-    'pickle_protocol': 2  # pickle.HIGHEST_PROTOCOL (`2` is used to ensure Python 3-->2 compatibility)
+    'pickle_protocol': 4
 })
 
 
