@@ -1,6 +1,8 @@
 from __future__ import print_function, division, unicode_literals
 
-import os
+import os, sys
+sys.path.insert(0,os.path.split(os.path.split(os.path.split(os.path.abspath(__file__))[0])[0])[0])
+
 import numpy
 from matplotlib import pyplot
 from pprint import pprint
